# -*- encoding: utf-8 -*-
import pilas
from pilas.actores import Actor

class Globo(Actor):
    "Representa un cuadro de dialogo estilo historietas."

    def __init__(self, texto, x=0, y=0, dialogo=None, avance_con_clicks=True, autoeliminar=False):
        self.dialogo = dialogo
        Actor.__init__(self, imagen='invisible.png', x=x, y=y)

        ancho, alto = pilas.utils.obtener_area_de_texto(texto)

        ancho = int((ancho + 12) - (ancho % 12))
        alto = int((alto + 12) - alto % 12)

        self.imagen = pilas.imagenes.cargar_superficie(ancho + 36, alto + 24 + 35)

        self._pintar_globo(ancho, alto)
        self.imagen.texto(texto, 17, 30)
        self.centro = ("derecha", "abajo")
        self.escala = 0.1
        self.escala = [1], 0.2

        if avance_con_clicks:
            pilas.eventos.click_de_mouse.conectar(self.cuando_quieren_avanzar)

        if autoeliminar:
            pilas.mundo.tareas.una_vez(3, self.eliminar)

    def colocar_origen_del_globo(self, x, y):
        "Cambia la posicion del globo para que el punto de donde se emite el globo sea (x, y)."
        self.x = x
        self.y = y
    

    def cuando_quieren_avanzar(self, *k):
        if self.dialogo:
            self.dialogo.avanzar_al_siguiente_dialogo()
        else:
            self.eliminar()
            
    def _pintar_globo(self, ancho, alto):
        imagen = pilas.imagenes.cargar("globo.png")

        # esquina sup-izq
        self.imagen.pintar_parte_de_imagen(imagen, 0, 0, 12, 12, 0, 0)

        # borde superior
        for x in range(0, int(ancho) + 12, 12):
            self.imagen.pintar_parte_de_imagen(imagen, 12, 0, 12, 12, 12 + x, 0)
        
         # esquina sup-der
        self.imagen.pintar_parte_de_imagen(imagen, 100, 0, 12, 12, 12 + int(ancho) + 12, 0)

        # centro del dialogo
        for y in range(0, int(alto) + 12, 12):
            # borde izquierdo
            self.imagen.pintar_parte_de_imagen(imagen, 0, 12, 12, 12, 0, 12 + y)
            # linea horizontal blanca, para el centro del dialogo.
            for x in range(0, int(ancho) + 12, 12):
                self.imagen.pintar_parte_de_imagen(imagen, 12, 12, 12, 12, 12 + x, 12 + y)
            
            # borde derecho
            self.imagen.pintar_parte_de_imagen(imagen, 100, 12, 12, 12, 12 + int(ancho) + 12, 12 + y)
        
        # parte inferior
        self.imagen.pintar_parte_de_imagen(imagen, 0, 35, 12, 12, 0, 0 + int(alto) + 12 + 12)

        # linea horizontal de la parte inferior
        for x in range(0, int(ancho) + 12, 12):
            self.imagen.pintar_parte_de_imagen(imagen, 12, 35, 12, 12, 12 + x, 0 + int(alto) + 12 + 12)
        
        self.imagen.pintar_parte_de_imagen(imagen, 100, 35, 12, 12, 12 + int(ancho) + 12, 0 + int(alto) + 12 + 12)
        # Pico de la parte de abajo
        self.imagen.pintar_parte_de_imagen(imagen, 67, 35, 33, 25, int(ancho) - 12, 0 + int(alto) + 12 + 12)

    def eliminar(self):
        Actor.eliminar(self)



# -*- encoding: utf-8 -*-
# Pilas engine - A video game framework.
#
# Copyright 2010 - Hugo Ruscitti
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar

import pilas

class Dialogo:
    "Representa una secuencia de mensajes entre varios actores."
    
    def __init__(self, modo_automatico=True):
        self.dialogo = []
        self.dialogo_actual = None
        self.modo_automatico = modo_automatico
    
    def decir(self, actor, texto):
        self.dialogo.append((actor, texto))

    def decir_inmediatamente(self, actor, texto):
        self.dialogo = []
        self._eliminar_dialogo_actual()
        self.decir(actor, texto)
        siguiente = self.obtener_siguiente_dialogo_o_funcion()
        self._mostrar_o_ejecutar_siguiente(siguiente)

    def elegir(self, actor, texto, opciones, funcion_a_invocar):
        self.dialogo.append((actor, texto, opciones, funcion_a_invocar))
        
    def ejecutar(self, funcion):
        self.dialogo.append(funcion)

    def iniciar(self):
        self.avanzar_al_siguiente_dialogo()

    def obtener_siguiente_dialogo_o_funcion(self):
        if self.dialogo:
            return self.dialogo.pop(0)
        
    def _eliminar_dialogo_actual(self):
        if self.dialogo_actual:
            self.dialogo_actual.eliminar()
            self.dialogo_actual = None

    def _mostrar_o_ejecutar_siguiente(self, siguiente):
        if isinstance(siguiente, tuple):
            # Es un mensaje de dialogo simple
            if len(siguiente) == 2:
                actor, texto = siguiente
                self.dialogo_actual = pilas.actores.Globo(texto, dialogo=self, avance_con_clicks=self.modo_automatico)
            else:
                # Es un mensaje con seleccion.
                actor, texto, opciones, funcion_a_invocar = siguiente
                self.dialogo_actual = pilas.actores.GloboElegir(texto, opciones, funcion_a_invocar, dialogo=self)
                
            self.dialogo_actual.colocar_origen_del_globo(actor.x, actor.arriba)
        else:
            siguiente()
            self.avanzar_al_siguiente_dialogo()

    def avanzar_al_siguiente_dialogo(self, evento=None):
        self._eliminar_dialogo_actual()
        siguiente = self.obtener_siguiente_dialogo_o_funcion()
        
        if siguiente:
            self._mostrar_o_ejecutar_siguiente(siguiente)
        else:
            return False
            
        return True



# -*- encoding: utf-8 -*-
# pilas engine - a video game framework.
#
# copyright 2010 - hugo ruscitti
# license: lgplv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# website - http://www.pilas-engine.com.ar

import pilas

class Escena(object):
    "Escena abstracta."

    def __init__(self):
        pilas.actores.utils.destruir_a_todos()
        pilas.mundo.definir_escena(self)

    def iniciar(self):
        pass

    def terminar(self):
        pass


class Normal(Escena):
    "Representa la escena inicial mas simple."

    def __init__(self, color_de_fondo=None):
        Escena.__init__(self)
        self.fondo = pilas.fondos.Color(color_de_fondo)



import pilas
import piedra_espacial
import random

class Estado:
    "Representa una escena dentro del juego."

    def actualizar(self):
        raise Exception("Tienes que sobrescribir este metodo...")

class Jugando(Estado):
    "Representa el estado de juego."

    def __init__(self, juego, nivel):
        self.nivel = nivel
        self.juego = juego
        self.juego.crear_naves(cantidad=nivel * 3)

        # Cada un segundo le avisa al estado que cuente.
        pilas.mundo.agregar_tarea(1, self.actualizar)

    def actualizar(self):
        if self.juego.ha_eliminado_todas_las_piedras():
            self.juego.cambiar_estado(Iniciando(self.juego, self.nivel + 1))
            return False

        return True

class Iniciando(Estado):
    "Estado que indica que el juego ha comenzado."

    def __init__(self, juego, nivel):
        self.texto = pilas.actores.Texto("Iniciando el nivel %d" %(nivel))
        self.nivel = nivel
        self.texto.color = pilas.colores.blanco
        self.contador_de_segundos = 0
        self.juego = juego

        # Cada un segundo le avisa al estado que cuente.
        pilas.mundo.agregar_tarea(1, self.actualizar)

    def actualizar(self):
        self.contador_de_segundos += 1

        if self.contador_de_segundos > 2:
            self.juego.cambiar_estado(Jugando(self.juego, self.nivel))
            self.texto.eliminar()
            return False

        return True    # para indicarle al contador que siga trabajado.

class PierdeVida(Estado):

    def __init__(self, juego):
        self.contador_de_segundos = 0
        self.juego = juego

        if self.juego.contador_de_vidas.le_quedan_vidas():
            self.juego.contador_de_vidas.quitar_una_vida()
            pilas.mundo.agregar_tarea(1, self.actualizar)
        else:
            juego.cambiar_estado(PierdeTodoElJuego(juego))

    def actualizar(self):
        self.contador_de_segundos += 1

        if self.contador_de_segundos > 2:
            self.juego.crear_nave()
            return False

        return True

class PierdeTodoElJuego(Estado):

    def __init__(self, juego):
        # Muestra el mensaje "has perdido"
        mensaje = pilas.actores.Texto("Lo siento, has perdido!")
        mensaje.color=pilas.colores.blanco
        mensaje.abajo = 240
        mensaje.abajo = [-20]

    def actualizar(self):
        pass


class Juego(pilas.escenas.Escena):
    "Es la escena que permite controlar la nave y jugar"

    def __init__(self):
        pilas.escenas.Escena.__init__(self)
        pilas.fondos.Espacio()
        pilas.eventos.pulsa_tecla_escape.conectar(self.cuando_pulsa_tecla_escape)
        self.piedras = []
        self.crear_nave()
        self.crear_contador_de_vidas()
        self.cambiar_estado(Iniciando(self, 1))
        self.puntaje = pilas.actores.Puntaje(x=280, y=220, color=pilas.colores.blanco)

    def cambiar_estado(self, estado):
        self.estado = estado

    def crear_nave(self):
        nave = pilas.actores.Nave()
        nave.aprender(pilas.habilidades.SeMantieneEnPantalla)
        nave.definir_enemigos(self.piedras, self.cuando_explota_asterioide)
        pilas.mundo.colisiones.agregar(nave, self.piedras, self.explotar_y_terminar)

    def cuando_explota_asterioide(self):
        self.puntaje.aumentar(1)

    def crear_contador_de_vidas(self):
        import contador_de_vidas
        self.contador_de_vidas = contador_de_vidas.ContadorDeVidas(3)

    def cuando_pulsa_tecla_escape(self, *k, **kv):
        "Regresa al menu principal."
        import escena_menu
        escena_menu.EscenaMenu()

    def explotar_y_terminar(self, nave, piedra):
        "Responde a la colision entre la nave y una piedra."
        nave.eliminar()

        self.cambiar_estado(PierdeVida(self))

    def crear_naves(self, cantidad):
        "Genera una cantidad especifica de naves en el escenario."
        fuera_de_la_pantalla = [-600, -650, -700, -750, -800]
        tamanos = ['grande', 'media', 'chica']

        for x in range(cantidad):
            x = random.choice(fuera_de_la_pantalla)
            y = random.choice(fuera_de_la_pantalla)
            t = random.choice(tamanos)

            piedra_nueva = piedra_espacial.PiedraEspacial(self.piedras, x=x, y=y, tamano=t)
            self.piedras.append(piedra_nueva)

    def ha_eliminado_todas_las_piedras(self):
        return len(self.piedras) == 0



import pilas
# Permite que este ejemplo funcion incluso si no has instalado pilas.
import sys
sys.path.insert(0, "..")


pilas.iniciar()

mono = pilas.actores.Mono()

otro = mono.duplicar(y=-150)
pilas.avisar("Duplicando actores.")
pilas.ejecutar()



import pilas
# Permite que este ejemplo funcion incluso si no has instalado pilas.
import sys
sys.path.insert(0, "..")

pilas.iniciar()
mono = pilas.actores.Mono()
mono.aprender(pilas.habilidades.MoverseConElTeclado)

pilas.avisar("Use los direccionales del teclado para mover al mono.")
pilas.ejecutar()



# -*- encoding: utf-8 -*-
# Permite que este ejemplo funcion incluso si no has instalado pilas.
import sys
sys.path.insert(0, "..")
import pilas

pilas.iniciar()

grilla = pilas.imagenes.cargar_grilla("explosion.png", 7)
animacion = pilas.actores.Animacion(grilla, True, velocidad=1)

texto = pilas.actores.Texto("1 cuadro por segundo", x=100, y=80, magnitud=10)
texto.color = pilas.colores.negro

def cambia_velocidad(progreso):
    progreso = 1 + progreso * 60.0
    animacion.velocidad = progreso
    texto.texto = str(int(progreso)) + " cuadros por segundo"


barra = pilas.interfaz.Deslizador(y=100)
barra.conectar(cambia_velocidad)


pilas.avisar("Usa la barra para modificar la velocidad de la animacion.")
pilas.ejecutar()



# -*- encoding: utf-8 -*-
# Pilas engine - A video game framework.
#
# Copyright 2010 - Hugo Ruscitti
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar

class Tarea(object):

    def __init__(self, time_out, dt, funcion, parametros, una_vez):
        """

        Parametros:

            - time_out: el tiempo absoluto para ejecutar la tarea.
            - dt: la frecuencia de ejecución.
            - funcion: la funcion a invocar.
            - parametros: una lista de argumentos para la funcion anterior.
            - una_vez: indica si la funcion se tiene que ejecutar una sola vez.
        """

        self.time_out = time_out
        self.dt = dt
        self.funcion = funcion
        self.parametros = parametros
        self.una_vez = una_vez
        self.activa = True

    def ejecutar(self):
        return self.funcion(*self.parametros)

    def eliminar(self):
        self.activa = False

class TareaCondicional(Tarea):

    def ejecutar(self):
        retorno = Tarea.ejecutar(self)

        if not retorno:
            self.una_vez = True

class Tareas(object):
    """Contenedor de tareas a ejecutar por tiempo.

    El Tareas es un planificador de tareas, permite que
    podamos ejecutar funciones y métodos luego de transcurrido
    el tiempo que queramos.

    Por ejemplo, si se quiere que el planificardor ejecute
    una función dentro de dos segundos podemos escribir:

        pilas.mundo.agregar_tarea(2, hola)

    o bien, especificando argumentos para esa función:

        pilas.mundo.agregar_tarea(4, hola, 'persona')

    La función que se especifique como segundo argumento
    tiene que retornar True o False. Si retorna True será
    colocada nuevamente en la cola de tareas una vez que se
    ejecute (esto es útil para crear bucles).
    """

    def __init__(self):
        self.tareas_planificadas = []
        self.contador_de_tiempo = 0

    def actualizar(self, dt):
        "Actualiza los contadores de tiempo y ejecuta las tareas pendientes."
        self.contador_de_tiempo += dt
        to_remove = []
         
        for tarea in self.tareas_planificadas:
            if self.contador_de_tiempo > tarea.time_out:
                if tarea.activa:
                    tarea.ejecutar()
                
                    if tarea.una_vez:
                        self.tareas_planificadas.remove(tarea)
                    else:
                        w = self.contador_de_tiempo - tarea.time_out
                        parte_entera = int((w)/float(tarea.dt))
                        resto = w - (parte_entera * tarea.dt)

                        for x in range(parte_entera):
                            tarea.ejecutar()

                        tarea.time_out += tarea.dt + (parte_entera * tarea.dt) - resto
                else:
                    self.tareas_planificadas.remove(tarea)

    def _agregar(self, tarea):
        "Agrega una nueva tarea para ejecutarse luego."
        self.tareas_planificadas.append(tarea)

    def una_vez(self, time_out, function, params=[]):
        tarea = Tarea(self.contador_de_tiempo + time_out, time_out, function, params, True)
        self._agregar(tarea)
        return tarea

    def siempre(self, time_out, function, params=[]):
        tarea = Tarea(self.contador_de_tiempo + time_out, time_out, function, params, False)
        self._agregar(tarea)
        return tarea

    def condicional(self, time_out, function, params=[]):
        tarea = TareaCondicional(self.contador_de_tiempo + time_out, time_out, function, params, False)
        self._agregar(tarea)
        return tarea



# -*- encoding: utf-8 -*-
# pilas engine - a video game framework.
#
# copyright 2010 - hugo ruscitti
# license: lgplv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# website - http://www.pilas-engine.com.ar

import os
import sys
import copy
from PyQt4 import QtGui, QtCore
from PyQt4.QtGui import QWidget

try:
    from PyQt4 import QtOpenGL
    from PyQt4.QtOpenGL import QGLWidget
except ImportError:
    QGLWidget = object
    print "No se encuentra soporte OpenGL en este equipo."


import motor
from pilas import imagenes
from pilas import actores
from pilas import eventos
from pilas import utils
from pilas import depurador

from pilas import fps
from pilas import simbolos
from pilas import colores


class BaseActor(object):
    
    def __init__(self):
        self._rotacion = 0
        self._escala = 1
        self._transparencia = 0
        self.centro_x = 0
        self.centro_y = 0
        self._espejado = False
        self.fijo = 0

    def definir_centro(self, x, y):
        self.centro_x = x
        self.centro_y = y

    def obtener_posicion(self):
        return self.x, self.y

    def definir_posicion(self, x, y):
        self.x, self.y = x, y

    def obtener_escala(self):
        return self._escala

    def definir_escala(self, s):
        self._escala = s

    def definir_transparencia(self, nuevo_valor):
        self._transparencia = nuevo_valor

    def obtener_transparencia(self):
        return self._transparencia

    def obtener_rotacion(self):
        return self._rotacion

    def definir_rotacion(self, r):
        self._rotacion = r
        
    def set_espejado(self, espejado):        
        self._espejado = espejado
        
class QtImagen(object):

    def __init__(self, ruta):
        self.ruta_original = ruta
        self._imagen = QtGui.QPixmap(ruta)

    def ancho(self):
        return self._imagen.size().width()

    def alto(self):
        return self._imagen.size().height()

    def centro(self):
        "Retorna una tupla con la coordenada del punto medio del la imagen."
        return (self.ancho()/2, self.alto()/2)

    def avanzar(self):
        pass

    def dibujar(self, motor, x, y, dx=0, dy=0, escala_x=1, escala_y=1, rotacion=0, transparencia=0):
        """Dibuja la imagen sobre la ventana que muestra el motor.

           x, y: indican la posicion dentro del mundo.
           dx, dy: es el punto centro de la imagen (importante para rotaciones).
           escala_x, escala_yindican cambio de tamano (1 significa normal).
           rotacion: angulo de inclinacion en sentido de las agujas del reloj.
        """

        motor.canvas.save()
        centro_x, centro_y = motor.centro_fisico()
        motor.canvas.translate(x + centro_x, centro_y - y)
        motor.canvas.rotate(rotacion)
        motor.canvas.scale(escala_x, escala_y)

        if transparencia:
            motor.canvas.setOpacity(1 - transparencia/100.0)

        self._dibujar_pixmap(motor, -dx, -dy)
        motor.canvas.restore()

    def _dibujar_pixmap(self, motor, x, y):
        motor.canvas.drawPixmap(x, y, self._imagen)

    def __str__(self):
        nombre_imagen = os.path.basename(self.ruta_original)
        return "<Imagen del archivo '%s'>" %(nombre_imagen)


class QtGrilla(QtImagen):

    """Representa una grilla regular, que se utiliza en animaciones.

       La grilla regular se tiene que crear indicando la cantidad
       de filas y columnas. Una vez definida se puede usar como
       una imagen normal, solo que tiene dos metodos adicionales
       para ``definir_cuadro`` y ``avanzar`` el cuadro actual.
    """

    def __init__(self, ruta, columnas=1, filas=1):
        QtImagen.__init__(self, ruta)
        self.cantidad_de_cuadros = columnas * filas
        self.columnas = columnas
        self.filas = filas
        self.cuadro_ancho = QtImagen.ancho(self) / columnas
        self.cuadro_alto = QtImagen.alto(self) / filas
        self.definir_cuadro(0)

    def ancho(self):
        return self.cuadro_ancho

    def alto(self):
        return self.cuadro_alto

    def _dibujar_pixmap(self, motor, x, y):
        motor.canvas.drawPixmap(x, y, self._imagen, self.dx, self.dy, 
                self.cuadro_ancho, self.cuadro_alto)

    def definir_cuadro(self, cuadro):
        self._cuadro = cuadro

        frame_col = cuadro % self.columnas
        frame_row = cuadro / self.columnas

        self.dx = frame_col * self.cuadro_ancho
        self.dy = frame_row * self.cuadro_alto

    def avanzar(self):
        ha_reiniciado = False
        cuadro_actual = self._cuadro + 1

        if cuadro_actual >= self.cantidad_de_cuadros:
            cuadro_actual = 0
            ha_reiniciado = True

        self.definir_cuadro(cuadro_actual)
        return ha_reiniciado

    def obtener_cuadro(self):
        return self._cuadro

    def dibujarse_sobre_una_pizarra(self, pizarra, x, y):
        pizarra.pintar_parte_de_imagen(self, self.dx, self.dy, self.cuadro_ancho, self.cuadro_alto, x, y)

class QtTexto(QtImagen):

    def __init__(self, texto, magnitud, motor):
        self._ancho, self._alto = motor.obtener_area_de_texto(texto, magnitud)

    def _dibujar_pixmap(self, motor, dx, dy):
        nombre_de_fuente = motor.canvas.font().family()
        fuente = QtGui.QFont(nombre_de_fuente, self.magnitud)
        metrica = QtGui.QFontMetrics(fuente)

        r, g, b, a = self.color.obtener_componentes()
        motor.canvas.setPen(QtGui.QColor(r, g, b))
        motor.canvas.setFont(fuente)
        lines = self.texto.split('\n')

        for line in lines:
            motor.canvas.drawText(dx, dy + self._alto, line)
            dy += metrica.height()

    def ancho(self):
        return self._ancho

    def alto(self):
        return self._alto


class QtLienzo(QtImagen):

    def __init__(self):
        pass

    def texto(self, motor, cadena, x=0, y=0, magnitud=10, fuente=None, color=colores.negro):
        "Imprime un texto respespetando el desplazamiento de la camara."
        self.texto_absoluto(motor, cadena, x, y, magnitud, fuente, color)

    def texto_absoluto(self, motor, cadena, x=0, y=0, magnitud=10, fuente=None, color=colores.negro):
        "Imprime un texto sin respetar al camara."
        x, y = utils.hacer_coordenada_pantalla_absoluta(x, y)

        r, g, b, a = color.obtener_componentes()
        motor.canvas.setPen(QtGui.QColor(r, g, b))

        if not fuente:
            fuente = motor.canvas.font().family()

        motor.canvas.setFont(QtGui.QFont(fuente, magnitud))
        motor.canvas.drawText(x, y, cadena)

    def pintar(self, motor, color):
        r, g, b, a = color.obtener_componentes()
        ancho, alto = motor.obtener_area()
        motor.canvas.fillRect(0, 0, ancho, alto, QtGui.QColor(r, g, b))

    def linea(self, motor, x0, y0, x1, y1, color=colores.negro, grosor=1):
        x0, y0 = utils.hacer_coordenada_pantalla_absoluta(x0, y0)
        x1, y1 = utils.hacer_coordenada_pantalla_absoluta(x1, y1)

        r, g, b, a = color.obtener_componentes()
        color = QtGui.QColor(r, g, b)
        pen = QtGui.QPen(color, grosor)
        motor.canvas.setPen(pen)
        motor.canvas.drawLine(x0, y0, x1, y1)

    def poligono(self, motor, puntos, color=colores.negro, grosor=1, cerrado=False):
        x, y = puntos[0]
        if cerrado:
            puntos.append((x, y))

        for p in puntos[1:]:
            nuevo_x, nuevo_y = p
            self.linea(motor, x, y, nuevo_x, nuevo_y, color, grosor)
            x, y = nuevo_x, nuevo_y


    def cruz(self, motor, x, y, color=colores.negro, grosor=1):
        t = 3
        self.linea(motor, x - t, y - t, x + t, y + t, color, grosor)
        self.linea(motor, x + t, y - t, x - t, y + t, color, grosor)

    def circulo(self, motor, x, y, radio, color=colores.negro, grosor=1):
        x, y = utils.hacer_coordenada_pantalla_absoluta(x, y)

        r, g, b, a = color.obtener_componentes()
        color = QtGui.QColor(r, g, b)
        pen = QtGui.QPen(color, grosor)
        motor.canvas.setPen(pen)
        motor.canvas.drawEllipse(x -radio, y-radio, radio*2, radio*2)

    def rectangulo(self, motor, x, y, ancho, alto, color=colores.negro, grosor=1):
        x, y = utils.hacer_coordenada_pantalla_absoluta(x, y)

        r, g, b, a = color.obtener_componentes()
        color = QtGui.QColor(r, g, b)
        pen = QtGui.QPen(color, grosor)
        motor.canvas.setPen(pen)
        motor.canvas.drawRect(x, y, ancho, alto)

class QtSuperficie(QtImagen):

    def __init__(self, ancho, alto):
        self._imagen = QtGui.QPixmap(ancho, alto)
        self._imagen.fill(QtGui.QColor(255, 255, 255, 0))
        self.canvas = QtGui.QPainter()

    def pintar(self, color):
        r, g, b, a = color.obtener_componentes()
        self._imagen.fill(QtGui.QColor(r, g, b, a))

    def pintar_parte_de_imagen(self, imagen, origen_x, origen_y, ancho, alto, x, y):
        self.canvas.begin(self._imagen)
        self.canvas.drawPixmap(x, y, imagen._imagen, origen_x, origen_y, ancho, alto)
        self.canvas.end()

    def pintar_imagen(self, imagen, x=0, y=0):
        self.pintar_parte_de_imagen(imagen, 0, 0, imagen.ancho(), imagen.alto(), x, y)

    def texto(self, cadena, x=0, y=0, magnitud=10, fuente=None, color=colores.negro):
        self.canvas.begin(self._imagen)
        r, g, b, a = color.obtener_componentes()
        self.canvas.setPen(QtGui.QColor(r, g, b))
        dx = x
        dy = y

        if not fuente:
            fuente = self.canvas.font().family()

        font = QtGui.QFont(fuente, magnitud)
        self.canvas.setFont(font)
        metrica = QtGui.QFontMetrics(font)

        for line in cadena.split('\n'):
            self.canvas.drawText(dx, dy, line)
            dy += metrica.height()

        self.canvas.end()

    def circulo(self, x, y, radio, color=colores.negro, relleno=False, grosor=1):
        self.canvas.begin(self._imagen)

        r, g, b, a = color.obtener_componentes()
        color = QtGui.QColor(r, g, b)
        pen = QtGui.QPen(color, grosor)
        self.canvas.setPen(pen)

        if relleno:
            self.canvas.setBrush(color)

        self.canvas.drawEllipse(x -radio, y-radio, radio*2, radio*2)
        self.canvas.end()

    def rectangulo(self, x, y, ancho, alto, color=colores.negro, relleno=False, grosor=1):
        self.canvas.begin(self._imagen)

        r, g, b, a = color.obtener_componentes()
        color = QtGui.QColor(r, g, b)
        pen = QtGui.QPen(color, grosor)
        self.canvas.setPen(pen)

        if relleno:
            self.canvas.setBrush(color)

        self.canvas.drawRect(x, y, ancho, alto)
        self.canvas.end()

    def linea(self, x, y, x2, y2, color=colores.negro, grosor=1):
        self.canvas.begin(self._imagen)

        r, g, b, a = color.obtener_componentes()
        color = QtGui.QColor(r, g, b)
        pen = QtGui.QPen(color, grosor)
        self.canvas.setPen(pen)

        self.canvas.drawLine(x, y, x2, y2)
        self.canvas.end()

    def poligono(self, puntos, color, grosor, cerrado=False):
        x, y = puntos[0]

        if cerrado:
            puntos.append((x, y))

        for p in puntos[1:]:
            nuevo_x, nuevo_y = p
            self.linea(x, y, nuevo_x, nuevo_y, color, grosor)
            x, y = nuevo_x, nuevo_y

    def dibujar_punto(self, x, y, color=colores.negro):
        self.circulo(x, y, 3, color=color, relleno=True)

    def limpiar(self):
        self._imagen.fill(QtGui.QColor(0, 0, 0, 0))

class QtActor(BaseActor):

    def __init__(self, imagen="sin_imagen.png", x=0, y=0):

        if isinstance(imagen, str):
            self.imagen = imagenes.cargar(imagen)
        else:
            self.imagen = imagen

        self.x = x
        self.y = y
        BaseActor.__init__(self)

    def definir_imagen(self, imagen):
        # permite que varios actores usen la misma grilla.
        if isinstance(imagen, QtGrilla):
            self.imagen = copy.copy(imagen)
        else:
            self.imagen = imagen

    def obtener_imagen(self):
        return self.imagen

    def dibujar(self, motor):
        escala_x, escala_y = self._escala, self._escala

        if self._espejado:
            escala_x *= -1

        if not self.fijo:
            x = self.x - motor.camara_x
            y = self.y - motor.camara_y
        else:
            x = self.x
            y = self.y

        self.imagen.dibujar(motor, x, y,
                self.centro_x, self.centro_y,
                escala_x, escala_y, self._rotacion, self._transparencia)

class QtSonido:

    def __init__(self, ruta):
        import pygame
        pygame.mixer.init()
        pygame.mixer.init()
        self.sonido = pygame.mixer.Sound(ruta)

    def reproducir(self):
        # TODO: quitar esta nota...
        # print "Usando pygame para reproducir sonido"
        self.sonido.play()
        
class QtBase(motor.Motor):
    
    app = QtGui.QApplication([])
    
    def __init__(self):
        motor.Motor.__init__(self)
        self.canvas = QtGui.QPainter()
        self.setMouseTracking(True)
        self.fps = fps.FPS(60, True)
        self.pausa_habilitada = False
        self.depurador = depurador.Depurador(self.obtener_lienzo(), self.fps)
        self.mouse_x = 0
        self.mouse_y = 0
        self.camara_x = 0
        self.camara_y = 0

    def iniciar_ventana(self, ancho, alto, titulo, pantalla_completa):
        self.ancho = ancho
        self.alto = alto
        self.ancho_original = ancho
        self.alto_original = alto
        self.titulo = titulo
        self.centrar_ventana()
        self.setWindowTitle(self.titulo)

        if pantalla_completa:
            self.showFullScreen()
        else:
            self.show()

        # Activa la invocacion al evento timerEvent.
        self.startTimer(1000/60.0)

    def pantalla_completa(self):
        self.showFullScreen()

    def pantalla_modo_ventana(self):
        self.showNormal()

    def esta_en_pantalla_completa(self):
        return self.isFullScreen()

    def alternar_pantalla_completa(self):
        """Permite cambiar el modo de video.

        Si está en modo ventana, pasa a pantalla completa y viceversa.
        """
        if self.esta_en_pantalla_completa():
            self.pantalla_modo_ventana()
        else:
            self.pantalla_completa()

    def centro_fisico(self):
        "Centro de la ventana para situar el punto (0, 0)"
        return self.ancho_original/2, self.alto_original/2

    def obtener_area(self):
        return (self.ancho_original, self.alto_original)

    def centrar_ventana(self):
        escritorio = QtGui.QDesktopWidget().screenGeometry()
        self.setGeometry(
                    (escritorio.width()-self.ancho)/2, 
                    (escritorio.height()-self.alto)/2, self.ancho, self.alto)

    def obtener_actor(self, imagen, x, y):
        return QtActor(imagen, x, y)

    def obtener_texto(self, texto, magnitud):
        return QtTexto(texto, magnitud, self)

    def obtener_grilla(self, ruta, columnas, filas):
        return QtGrilla(ruta, columnas, filas)

    def actualizar_pantalla(self):
        self.ventana.update()

    def definir_centro_de_la_camara(self, x, y):
        self.camara_x = x
        self.camara_y = y

    def obtener_centro_de_la_camara(self):
        return (self.camara_x, self.camara_y)

    def cargar_sonido(self, ruta):
        return QtSonido(ruta)

    def cargar_imagen(self, ruta):
        return QtImagen(ruta)

    def obtener_lienzo(self):
        return QtLienzo()

    def obtener_superficie(self, ancho, alto):
        return QtSuperficie(ancho, alto)

    def ejecutar_bucle_principal(self, mundo, ignorar_errores):
        sys.exit(self.app.exec_())

    def paintEvent(self, event):
        self.canvas.begin(self)

        self.canvas.setClipping(True)
        self.canvas.setClipRect(0, 0, self.alto * self.ancho_original / self.alto_original, self.alto)

        alto = self.alto / float(self.alto_original)
        self.canvas.scale(alto, alto)


        self.canvas.setRenderHint(QtGui.QPainter.HighQualityAntialiasing, False)
        self.canvas.setRenderHint(QtGui.QPainter.SmoothPixmapTransform, True)
        self.canvas.setRenderHint(QtGui.QPainter.Antialiasing, False)

        self.depurador.comienza_dibujado(self)

        for actor in actores.todos:
            try:
                actor.dibujar(self)
            except Exception as e:
                print e
                actor.eliminar()

            self.depurador.dibuja_al_actor(self, actor)

        self.depurador.termina_dibujado(self)
        self.canvas.end()

    def timerEvent(self, event):

        if not self.pausa_habilitada:
            try:
                self.realizar_actualizacion_logica()
            except Exception as e:
                print e

        # Invoca el dibujado de la pantalla.
        self.update()


    def realizar_actualizacion_logica(self):
        for x in range(self.fps.actualizar()):
            if not self.pausa_habilitada:
                eventos.actualizar.send("Qt::timerEvent")

                for actor in actores.todos:
                    actor.pre_actualizar()
                    actor.actualizar()

    def resizeEvent(self, event):
        self.ancho = event.size().width()
        self.alto = event.size().height()

    def mousePressEvent(self, e):
        escala = self.escala()
        x, y = utils.convertir_de_posicion_fisica_relativa(e.pos().x()/escala, e.pos().y()/escala)
        eventos.click_de_mouse.send("Qt::mousePressEvent", x=x, y=y, dx=0, dy=0)

    def mouseReleaseEvent(self, e):
        escala = self.escala()
        x, y = utils.convertir_de_posicion_fisica_relativa(e.pos().x()/escala, e.pos().y()/escala)
        eventos.termina_click.send("Qt::mouseReleaseEvent", x=x, y=y, dx=0, dy=0)

    def wheelEvent(self, e):
        eventos.mueve_rueda.send("ejecutar", delta=e.delta() / 120)

    def mouseMoveEvent(self, e):
        escala = self.escala()
        x, y = utils.convertir_de_posicion_fisica_relativa(e.pos().x()/escala, e.pos().y()/escala)
        dx, dy = x - self.mouse_x, y - self.mouse_y
        eventos.mueve_mouse.send("Qt::mouseMoveEvent", x=x, y=y, dx=dx, dy=dy)
        self.mouse_x = x
        self.mouse_y = y

    def keyPressEvent(self, event):
        codigo_de_tecla = self.obtener_codigo_de_tecla_normalizado(event.key())

        if event.key() == QtCore.Qt.Key_Escape:
            eventos.pulsa_tecla_escape.send("Qt::keyPressEvent")
        if event.key() == QtCore.Qt.Key_P:
            self.alternar_pausa()
        if event.key() == QtCore.Qt.Key_F:
            self.alternar_pantalla_completa()

        eventos.pulsa_tecla.send("Qt::keyPressEvent", codigo=codigo_de_tecla, texto=event.text())

    def keyReleaseEvent(self, event):
        codigo_de_tecla = self.obtener_codigo_de_tecla_normalizado(event.key())
        eventos.suelta_tecla.send("Qt::keyReleaseEvent", codigo=codigo_de_tecla, texto=event.text())

    def obtener_codigo_de_tecla_normalizado(self, tecla_qt):
        teclas = {
            QtCore.Qt.Key_Left: simbolos.IZQUIERDA,
            QtCore.Qt.Key_Right: simbolos.DERECHA,
            QtCore.Qt.Key_Up: simbolos.ARRIBA,
            QtCore.Qt.Key_Down: simbolos.ABAJO,
            QtCore.Qt.Key_Space: simbolos.SELECCION,
            QtCore.Qt.Key_Return: simbolos.SELECCION,
            QtCore.Qt.Key_F1: simbolos.F1,
            QtCore.Qt.Key_F2: simbolos.F2,
            QtCore.Qt.Key_F3: simbolos.F3,
            QtCore.Qt.Key_F4: simbolos.F4,
            QtCore.Qt.Key_F5: simbolos.F5,
            QtCore.Qt.Key_F6: simbolos.F6,
            QtCore.Qt.Key_F7: simbolos.F7,
            QtCore.Qt.Key_F8: simbolos.F8,
            QtCore.Qt.Key_F9: simbolos.F9,
            QtCore.Qt.Key_F10: simbolos.F10,
            QtCore.Qt.Key_F11: simbolos.F11,
            QtCore.Qt.Key_F12: simbolos.F12,
        }

        if teclas.has_key(tecla_qt):
            return teclas[tecla_qt]
        else:
            return tecla_qt

    def escala(self):
        "Obtiene la proporcion de cambio de escala de la pantalla"
        return self.alto / float(self.alto_original)

    def obtener_area_de_texto(self, texto, magnitud=10):
        ancho = 0
        alto = 0

        fuente = QtGui.QFont()
        fuente.setPointSize(magnitud)
        metrica = QtGui.QFontMetrics(fuente)

        lineas = texto.split('\n')

        for linea in lineas:
            ancho = max(ancho, metrica.width(linea))
            alto += metrica.height()

        return ancho, alto

    def alternar_pausa(self):
        if self.pausa_habilitada:
            self.pausa_habilitada = False
            self.actor_pausa.eliminar()
        else:
            self.pausa_habilitada = True
            self.actor_pausa = actores.Pausa()

    def ocultar_puntero_del_mouse(self):
        bitmap = QtGui.QBitmap(1, 1)
        nuevo_cursor = QtGui.QCursor(bitmap, bitmap)
        self.setCursor(QtGui.QCursor(nuevo_cursor))

class Qt(QtBase, QWidget):

    def __init__(self):
        QWidget.__init__(self)
        QtBase.__init__(self)

class QtGL(QtBase, QGLWidget):

    def __init__(self):
        if not QGLWidget:
            print "Lo siento, OpenGL no esta disponible..."

        QGLWidget.__init__(self)
        QtBase.__init__(self)
        self._pintar_fondo_negro()

    def _pintar_fondo_negro(self):
        color = QtGui.QColor(99, 0, 0)
        self.setStyleSheet("QWidget { background-color: %s }" % color.name())

if QGLWidget == object:
    QtGL = Qt



# -*- encoding: utf-8 -*-
import pilas

pilas.iniciar()

# La secuencia es sencilla, el mono dice 'hola', 
mono = pilas.actores.Mono(x=-100)
mono_chiquito = pilas.actores.Mono(x=200)
mono_chiquito.escala = 0.75

d = pilas.actores.Dialogo()
d.decir(mono, "Hola, como estas?")
d.decir(mono_chiquito, "Bien, ¿y vos?...")
d.decir(mono, "Bien... ¡Mirá cómo salto !")

def hacer_que_el_mono_salte():
    mono.sonreir()
    mono.y = [300], 1

d.ejecutar(hacer_que_el_mono_salte)
d.decir(mono_chiquito, "wow...")

d.iniciar()
pilas.ejecutar()




# -*- encoding: utf-8 -*-
# pilas engine - a video game framework.
#
# copyright 2010 - hugo ruscitti
# license: lgplv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# website - http://www.pilas-engine.com.ar
#
# Deslizador creado por Pablo Garrido

import pilas
from pilas.actores import Actor


class Deslizador(Actor):

    def __init__(self, x=0, y=0, ruta_barra = 'interfaz/barra.png',
                                 ruta_deslizador = 'interfaz/deslizador.png'):

        self.deslizador = None
        Actor.__init__(self, ruta_barra, x=x, y=y)
        self.deslizador = Actor(ruta_deslizador, self.x, self.y)
        self.centro = ('izquierda', 'centro')
        
        self.click = False
        
        pilas.eventos.click_de_mouse.conectar(self.click_del_mouse)
        pilas.eventos.mueve_mouse.conectar(self.movimiento_del_mouse)
        pilas.eventos.termina_click.conectar(self.termino_del_click)

        self.progreso = 0
        self.posicion_relativa_x = 0

        self.funciones = []

        # establecemos posicion inicial
        self.x = x
        self.y = y

    def set_transparencia(self, nuevo_valor):
        self.transparencia = nuevo_valor
        self.deslizador.transparencia = nuevo_valor

    def definir_posicion(self, x, y):
        self.limite_izq = self.x
        self.limite_der = self.x + self.obtener_ancho()

        self._actor.definir_posicion(x, y)
        if self.deslizador:
            self.deslizador.definir_posicion(x + self.posicion_relativa_x, y)

    def conectar(self, f):
        self.funciones.append(f)        

    def desconectar(self, f):
        self.funciones.remove(f)

    def ejecutar_funciones(self, valor):
        for i in self.funciones:
            i(valor)
    
    def click_del_mouse(self, click):
 
        if self.deslizador.colisiona_con_un_punto(click.x, click.y):
            self.click = True

    def movimiento_del_mouse(self, movimiento):
        if self.click == True:
            ancho = self.obtener_ancho()
            factor = (self.deslizador.x + (ancho - abs(self.x))) / ancho - 1
            self.progreso = factor
            
            self.ejecutar_funciones(factor)

            self.deslizador.x = movimiento.x

            if self.deslizador.x <= self.limite_izq:
                self.deslizador.x = self.limite_izq

            elif self.deslizador.x >= self.limite_der:
                self.deslizador.x = self.limite_der

            self.posicion_relativa_x = self.deslizador.x - self.x
                

    def termino_del_click(self, noclick):
        self.click = False



# -*- encoding: utf-8 -*-
# pilas engine - a video game framework.
#
# copyright 2010 - hugo ruscitti
# license: lgplv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# website - http://www.pilas-engine.com.ar
#
# Deslizador creado por Pablo Garrido

import pilas

class Selector(pilas.actores.Actor):
    
    def __init__(self, texto, x=0, y=0, ancho=200):
        pilas.actores.Actor.__init__(self, x=x, y=y)
        
        self.texto = texto
        self._cargar_lienzo(ancho)
        self._cargar_imagenes()
        self.funcion_de_respuesta = None

        self.deseleccionar()
        pilas.eventos.click_de_mouse.conectar(self.detection_click_mouse)

    def _cargar_imagenes(self):
        self.imagen_selector = pilas.imagenes.cargar("interfaz/selector.png")
        self.imagen_selector_seleccionado = pilas.imagenes.cargar("interfaz/selector_seleccionado.png")

    def _cargar_lienzo(self, ancho):
        self.imagen = pilas.imagenes.cargar_superficie(ancho, 29)
        
    def pintar_texto(self):
        self.imagen.texto(self.texto, 35, 20)
        
    def deseleccionar(self):
        self.seleccionado = False
        self.imagen.limpiar()
        self.imagen.pintar_imagen(self.imagen_selector)
        self.pintar_texto()
        self.centro = ("centro", "centro")
        
    def seleccionar(self):
        self.seleccionado = True
        self.imagen.limpiar()
        self.imagen.pintar_imagen(self.imagen_selector_seleccionado)
        self.pintar_texto()
        self.centro = ("centro", "centro")
                
    def detection_click_mouse(self, click):
        if self.colisiona_con_un_punto(click.x, click.y):
            self.alternar_seleccion()
                
    def alternar_seleccion(self):
        if self.seleccionado:
            self.deseleccionar()
        else:
            self.seleccionar()

        if self.funcion_de_respuesta:
            self.funcion_de_respuesta(self.seleccionado)

    def definir_accion(self, funcion):
        self.funcion_de_respuesta = funcion



# -*- encoding: utf-8 -*-
# pilas engine - a video game framework.
#
# copyright 2010 - hugo ruscitti
# license: lgplv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# website - http://www.pilas-engine.com.ar


from deslizador import Deslizador
from selector import Selector
from ingreso_de_texto import IngresoDeTexto
from lista_seleccion import ListaSeleccion
from boton import Boton



import pilas

# Permite que este ejemplo funcion incluso si no has instalado pilas.
import sys
sys.path.insert(0, "..")

pilas.iniciar()

sonido = pilas.sonidos.cargar("explosion.wav")

def reproducir_sonido_cuando_hace_click(evento):
    sonido.reproducir()
    
pilas.eventos.click_de_mouse.conectar(reproducir_sonido_cuando_hace_click)

pilas.avisar("Pulse sobre la pantalla para emitir un sonido de explosion.")
pilas.ejecutar()



import pilas

pilas.iniciar()

def selecciona_iniciar():
    pilas.avisar("Ha seleccionado Iniciar")

def selecciona_terminar():
    pilas.avisar("Ha seleccionado Terminar")


opciones = [
            ('Iniciar', selecciona_iniciar),
            ('Terminar', selecciona_terminar)]


pilas.avisar("Use el teclado para cambiar o seleccionar opciones.")
menu = pilas.actores.Menu(opciones)


pilas.ejecutar()



# -*- encoding: utf-8 -*-
# For pilas engine - a video game framework.
#
# copyright 2011 - Pablo Garrido
# license: lgplv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# website - http://www.pilas-engine.com.ar

import pilas

class EscenaMenu(pilas.escenas.Escena):
    "Escena de presentacion del juego."

    def __init__(self):
        pilas.actores.utils.eliminar_a_todos()
        pilas.fondos.Fondo('data/menu.png')
        self.titulo_de_juego()
        self.menu_de_juego()

    def titulo_de_juego(self):
        self.titulo = pilas.actores.Actor('data/titulo.png')
        
        self.titulo.y = 250
        
        self.titulo.y = [170], 0.8
    def menu_de_juego(self):
        
        opcion = pilas.actores.Actor('data/opcion.png')
        opcion.x = 146
        opcion.y = 26


        # animacion de cronometro
        class variable:
            intercambiar = True

        def animacion_escala():
            if variable.intercambiar == True:
                self.titulo.escala = [0.2]
                variable.intercambiar = False

            else:
                self.titulo.escala = [1.5]
                variable.intercambiar = True             

            return True

        pilas.mundo.agregar_tarea_siempre(0.2, animacion_escala)
        
        # boton inicio
        def presionamos_boton_inicio():
            self.iniciar_juego()
        
        def sobre_boton_inicio():
            self.boton_inicio.pintar_sobre()
            opcion.x = 146
            opcion.y = 26
            
            
        
        self.boton_inicio = pilas.actores.Boton(-550, 20, 'data/b_inicio.png', 'data/b_inicio_over.png','data/b_inicio_over.png')
        
        self.boton_inicio.conectar_presionado(presionamos_boton_inicio)
        self.boton_inicio.conectar_sobre(sobre_boton_inicio)
        self.boton_inicio.conectar_normal(self.boton_inicio.pintar_normal)
        
        self.boton_inicio.x = [-130]
        
        # boton ayuda
        def presionamos_boton_ayuda():
            self.mostrar_ayuda()
        
        def sobre_boton_ayuda():
            self.boton_ayuda.pintar_sobre()
            opcion.x = 90
            opcion.y = -52
        
        self.boton_ayuda = pilas.actores.Boton(-550, -53, 'data/b_ayuda.png', 'data/b_ayuda_over.png','data/b_ayuda_over.png')
        
        self.boton_ayuda.conectar_presionado(presionamos_boton_ayuda)
        self.boton_ayuda.conectar_sobre(sobre_boton_ayuda)
        self.boton_ayuda.conectar_normal(self.boton_ayuda.pintar_normal)
        
        self.boton_ayuda.x = [-160]
        
        # boton salir
        def presionamos_boton_salir():
            self.salir_del_juego()
        
        def sobre_boton_salir():            
            self.boton_salir.pintar_sobre()
            opcion.x = 21
            opcion.y = -119

        self.boton_salir = pilas.actores.Boton(-550, -126, 'data/b_salir.png', 'data/b_salir_over.png','data/b_salir_over.png')
        
        self.boton_salir.conectar_presionado(presionamos_boton_salir)
        self.boton_salir.conectar_sobre(sobre_boton_salir)
        self.boton_salir.conectar_normal(self.boton_salir.pintar_normal)
        
        self.boton_salir.x = [-190]

        
    def desactivar_botones(self):
        self.boton_inicio.desactivar()
        self.boton_ayuda.desactivar()
        self.boton_salir.desactivar()

            
    def iniciar_juego(self):
        import escena_juego
        escena_juego.Juego(nivel = 1)
        self.desactivar_botones()


    def salir_del_juego(self):
        pilas.terminar()

    def mostrar_ayuda(self):
        import escena_ayuda
        escena_ayuda.Ayuda()



import pilas
# Permite que este ejemplo funcion incluso si no has instalado pilas.
import sys
sys.path.insert(0, "..")

pilas.iniciar()
mono1 = pilas.actores.Mono()
mono2 = pilas.actores.Mono()

mono2.escala = 2
mono2.z = 100


mono1.aprender(pilas.habilidades.Arrastrable)

pilas.avisar("Hay dos actores, uno se puede arrastrar con el mouse.")
pilas.ejecutar()



# -*- encoding: utf-8 -*-
# pilas engine - a video game framework.
#
# copyright 2010 - hugo ruscitti
# license: lgplv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# website - http://www.pilas-engine.com.ar

import pytweener
from pilas import eventos
from pilas import tareas
from pilas import control
from pilas import fisica
from pilas import escenas
from pilas import colisiones
from pilas import camara


class Mundo(object):
    """Representa un objeto unico que mantiene en funcionamiento al motor.

    Mundo tiene como responsabilidad iniciar los componentes del
    motor y mantener el bucle de juego.
    """

    def __init__(self, motor, ancho, alto, titulo, fps=60, economico=True, 
            gravedad=(0, -90), pantalla_completa=False):
        self.motor = motor
        self.motor.iniciar_ventana(ancho, alto, titulo, pantalla_completa)

        self.tweener = pytweener.Tweener()
        self.tareas = tareas.Tareas() 
        self.control = control.Control()
        self.colisiones = colisiones.Colisiones()
        self.camara = camara.Camara(self)

        eventos.actualizar.conectar(self.actualizar_simuladores)
        self.fisica = fisica.Fisica(motor.obtener_area(), gravedad=gravedad)
        self.escena_actual = None

    def reiniciar(self):
        self.fisica.reiniciar()

    def actualizar_simuladores(self, evento):
        self.tweener.update(16)
        self.tareas.actualizar(1/60.0)
        self.fisica.actualizar()
        self.colisiones.verificar_colisiones()

    def terminar(self):
        import sys
        sys.exit(0)

    def ejecutar_bucle_principal(self, ignorar_errores=False):
        "Mantiene en funcionamiento el motor completo."
        self.motor.ejecutar_bucle_principal(self, ignorar_errores)

    def definir_escena(self, escena_nueva):
        "Cambia la escena que se muestra en pantalla"

        if self.escena_actual:
            self.escena_actual.terminar()

        self.escena_actual = escena_nueva
        escena_nueva.iniciar()

    def agregar_tarea_una_vez(self, time_out, function, *params): 
        return self.tareas.una_vez(time_out, function, params)

    def agregar_tarea_siempre(self, time_out, function, *params): 
        return self.tareas.siempre(time_out, function, params)

    def agregar_tarea(self, time_out, funcion, *parametros):
        return self.tareas.condicional(time_out, funcion, parametros)



import pilas

class GirarPorSiempre(pilas.habilidades.Habilidad):
    
    def __init__(self, receptor):
        self.receptor = receptor
    
    def actualizar(self):
        self.receptor.rotacion += 1

pilas.iniciar()

a = pilas.actores.Mono()
a.aprender(GirarPorSiempre)
a.aprender(GirarPorSiempre)
a.aprender(GirarPorSiempre)
pilas.ejecutar()



import pilas

pilas.iniciar()

mono = pilas.actores.Mono()

dialogo = pilas.actores.Dialogo()
dialogo.decir(mono, "Hola, podrias hacer click?")
dialogo.decir(mono, "Perfecto!!, gracias...")
dialogo.decir(mono, "Haciendo clicks avanzan los dialogos.")
dialogo.decir(mono, "eh ...")
dialogo.decir(mono, "m ...")
dialogo.decir(mono, "no se me ocurre que decir...")

dialogo.iniciar()


pilas.ejecutar()



import pilas

pilas.iniciar(usar_motor='qtgl')
ingreso_de_texto = pilas.interfaz.IngresoDeTexto("Ejemplo de transparencia")

def mueve_barra(grado):
    ingreso_de_texto.transparencia = grado * 100

b = pilas.interfaz.Deslizador(y=100)
b.conectar(mueve_barra)
        
pilas.avisar("Mueve el deslizador para cambiar la transparencia")
pilas.ejecutar()



#!/usr/bin/env python
import sys
from setuptools import setup
from setuptools import find_packages
from pilas import pilasversion

def error(biblioteca, web):
    print "Error, no se encuentra la biblioteca '%s' (de %s)" %(biblioteca, web)
    sys.exit(1)


try:
    import Box2D
except ImportError:
    error("box2d", "http://code.google.com/p/pybox2d")


setup(
        name='pilas',
        version=pilasversion.VERSION,
        description='A simple to use video game framework.',
        author='Hugo Ruscitti',
        author_email='hugoruscitti@gmail.com',
        install_requires=[
            'setuptools',
            ],
        packages=['pilas', 'pilas.actores', 'pilas.dispatch', 
                  'pilas.ejemplos', 'pilas.motores', 
                  'pilas.interfaz', 'pilas.video',
                  'pilas.cargador'],
        url='http://www.pilas-engine.com.ar',
        include_package_data = True,
        package_data = {
            'images': ['pilas/data/*', 'pilas/ejemplos/data/*', 
                       'pilas/data/fondos/*', 'pilas/data/juegobase/*', 
                       'pilas/data/juegobase/data/*',
                       'pilas/cargador/ejemplos',
                       'pilas/cargador/data'],
            },

        scripts=['bin/pilas'],

        classifiers = [
            'Intended Audience :: Developers',
            'Intended Audience :: Education',
            'License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)',
            'Natural Language :: Spanish',
            'Operating System :: OS Independent',
            'Programming Language :: Python',
            'Topic :: Games/Entertainment',
            'Topic :: Education',
            'Topic :: Software Development :: Libraries',
          ],
    )




# Permite que este ejemplo funcion incluso si no has instalado pilas.
import sys
sys.path.insert(0, "..")
import pilas
pilas.iniciar()

pilas.actores.CursorMano()
pilas.fondos.Pasto()
caja = pilas.actores.Caja() * 10
caja.aprender(pilas.habilidades.Arrastrable)

pelota = pilas.actores.Pelota() * 10
pelota.aprender(pilas.habilidades.Arrastrable)

pilas.avisar("Use el puntero del mouse para arrastrar las figuras.")
pilas.ejecutar()



# -*- encoding: utf-8 -*-
# Pilas engine - A video game framework.
#
# Copyright 2010 - Hugo Ruscitti
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar

import pilas
from pilas.actores import Animado
import copy

VELOCIDAD = 10


class Animacion(Animado):
    """Representa una animacion de una grilla de imagenes.

    Este actor toma una grilla de cuadros de animacion
    y los reproduce hasta que la animacion termina. Cuando
    la animacion termina se elimina a si mismo.
    """

    def __init__(self, grilla, ciclica=False, x=0, y=0, velocidad=VELOCIDAD):
        Animado.__init__(self, grilla, x=x, y=y)
        self.tick = 0
        self.ciclica = ciclica     # Indica si la animacion debe reiniciar luego de terminar.
        self.definir_velocidad_de_animacion(velocidad)

    def definir_velocidad_de_animacion(self, velocidad_de_animacion):
        self._velocidad_de_animacion = (1000.0 / 60) * velocidad_de_animacion

    def obtener_velocidad_de_animacion(self):
        return self._velocidad_de_animacion

    velocidad_de_animacion = property(obtener_velocidad_de_animacion, definir_velocidad_de_animacion)

    def actualizar(self):
        self.tick += self.velocidad_de_animacion

        if self.tick > 1000.0:
            self.tick -= 1000.0
            ha_reiniciado = self.imagen.avanzar()

            # Si la animacion ha terminado se elimina de la pantalla.
            if ha_reiniciado and not self.ciclica:
                self.eliminar()



# pilas engine - a video game framework.
#
# copyright 2010 - hugo ruscitti
# license: lgplv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# website - http://www.pilas-engine.com.ar


def abstract():
    raise Exception("Tienes que re-definir este metodo.")

class Motor(object):
    
    def __init__(self):
        pass

    def obtener_actor(self, imagen, x, y):
        abstract()

    def obtener_texto(self, texto, x, y):
        abstract()
    
    def obtener_canvas(self, ancho, alto):
        abstract()
    
    def obtener_grilla(self, ruta, columnas, filas):
        abstract()

    def crear_ventana(self, ancho, alto, titulo):
        abstract()

    def ocultar_puntero_del_mouse(self):
        abstract()

    def mostrar_puntero_del_mouse(self):
        abstract()

    def cerrar_ventana(self):
        abstract()

    def dibujar_circulo(self, x, y, radio, color, color_borde):
        abstract()

    def pulsa_tecla(self, tecla):
        abstract()

    def centrar_ventana(self):
        abstract()

    def procesar_y_emitir_eventos(self):
        abstract()

    def procesar_evento_teclado(self, event):
        abstract()

    def definir_centro_de_la_camara(self, x, y):
        abstract()

    def obtener_centro_de_la_camara(self):
        abstract()

    def pintar(self, color):
        abstract()

    def cargar_sonido(self, ruta):
        abstract()

    def cargar_imagen(self, ruta):
        abstract()

    def obtener_imagen_cairo(self, imagen):
        abstract()

    def ejecutar_bucle_principal(self, mundo, ignorar_errores):
        abstract()



# -*- encoding: utf-8 -*-
# Pilas engine - A video game framework.
#
# Copyright 2010 - Hugo Ruscitti
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar

import pilas
from pilas.actores import Actor

class Aceituna(Actor):

    def __init__(self, x=0, y=0):
        self.cuadro_normal = pilas.imagenes.cargar("aceituna.png")
        self.cuadro_reir = pilas.imagenes.cargar("aceituna_risa.png")
        self.cuadro_burla = pilas.imagenes.cargar("aceituna_burla.png")
        self.cuadro_grita = pilas.imagenes.cargar("aceituna_grita.png")

        Actor.__init__(self, x=x, y=y)
        self.imagen = self.cuadro_normal
        self.centro = ('centro', 'centro')
        self.radio_de_colision = 18

    def normal(self):
        self.imagen = self.cuadro_normal

    def reir(self):
        self.imagen = self.cuadro_reir

    def burlarse(self):
        self.imagen = self.cuadro_burla

    burlar = burlarse

    def gritar(self):
        self.imagen = self.cuadro_grita

    def saltar(self):
        self.hacer(pilas.comportamientos.Saltar())



# -*- encoding: utf-8 -*-
# Pilas engine - A video game framework.
#
# Copyright 2010 - Hugo Ruscitti
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar

import pilas.utils

class Estudiante:
    """Representa la habilidad de poder realizar habiliadades y comportamientos."""

    def __init__(self):
        self.habilidades = []
        self.comportamiento_actual = None
        self.comportamientos = []
        self.repetir_comportamientos_por_siempre = False

    def aprender(self, classname, *k, **w):
        "Comienza a realizar una habilidad indicada por parametros."
        objeto_habilidad = classname(self, *k, **w)
        self.habilidades.append(objeto_habilidad)

    def hacer_luego(self, comportamiento, repetir_por_siempre=False):
        """Define un nuevo comportamiento para realizar al final.
        
        Los actores pueden tener una cadena de comportamientos, este
        metodo agrega el comportamiento al final de la cadena.
        """
        
        self.comportamientos.append(comportamiento)
        self.repetir_comportamientos_por_siempre = repetir_por_siempre

    def hacer(self, comportamiento):
        "Define el comportamiento para el actor de manera inmediata."
        self.comportamientos.append(comportamiento)
        self._adoptar_el_siguiente_comportamiento()

    def eliminar_habilidades(self):
        "Elimina todas las habilidades asociadas al actor."
        for h in self.habilidades:
            h.eliminar()

    def eliminar_comportamientos(self):
        "Elimina todos los comportamientos que tiene que hacer el actor."
        for c in self.comportamientos:
            c.eliminar()

    def actualizar_habilidades(self):
        for h in self.habilidades:
            h.actualizar()

    def actualizar_comportamientos(self):
        termina = None

        if self.comportamiento_actual:
            termina = self.comportamiento_actual.actualizar()

            if termina:
                if self.repetir_comportamientos_por_siempre:
                    self.comportamientos.append(self.comportamiento_actual)
                self._adoptar_el_siguiente_comportamiento()
        else:
            self._adoptar_el_siguiente_comportamiento()

    def _adoptar_el_siguiente_comportamiento(self):
        if self.comportamientos:
            self.comportamiento_actual = self.comportamientos.pop(0)
            self.comportamiento_actual.iniciar(self)
        else:
            self.comportamiento_actual = None




import pilas
import random

class EscenaMenu(pilas.escenas.Escena):
    "Es la escena de presentacion donde se elijen las opciones del juego."

    def __init__(self):
        pilas.escenas.Escena.__init__(self)
        pilas.fondos.Fondo("data/menu.png")
        self.crear_titulo_del_juego()
        pilas.avisar("Use el teclado para controlar el menu.")
        self.crear_el_menu_principal()
        self.crear_asteroides()

    def crear_titulo_del_juego(self):
        logotipo = pilas.actores.Actor("data/titulo.png")
        logotipo.y = 300
        logotipo.y = [200]

    def crear_el_menu_principal(self):
        opciones = [
                    ("Comenzar a jugar", self.comenzar_a_jugar),
                    ("Ver ayuda", self.mostrar_ayuda_del_juego),
                    ("Salir", self.salir_del_juego),
                   ]
        self.menu = pilas.actores.Menu(opciones, y=-50)

    def comenzar_a_jugar(self):
        self.menu.desactivar()
        import escena_juego
        escena_juego.Juego()

    def mostrar_ayuda_del_juego(self):
        self.menu.desactivar()
        import escena_ayuda
        escena_ayuda.Ayuda()

    def salir_del_juego(self):
        pilas.terminar()

    def crear_asteroides(self):
        fuera_de_la_pantalla = [-600, -650, -700, -750, -800]
        import piedra_espacial
        for x in range(5):
            x = random.choice(fuera_de_la_pantalla)
            y = random.choice(fuera_de_la_pantalla)
            piedra_espacial.PiedraEspacial([], x=x, y=y, tamano="chica")



import pilas
# Permite que este ejemplo funcion incluso si no has instalado pilas.
import sys
sys.path.insert(0, "..")

pilas.iniciar()

grupo = pilas.atajos.fabricar(pilas.actores.Mono, 20)
grupo.aprender(pilas.habilidades.RebotarComoPelota)
grupo.sonreir()
pilas.ejecutar()

